@@ -33,9 +33,7 @@ import ta.volume
 import talib
 from plotly.graph_objs import Line
 from plotly.subplots import make_subplots
-from pykalman import KalmanFilter
 from quick_trade import utils
-from scipy import signal
 
 
 class Trader(object):
@@ -162,44 +160,6 @@ class Trader(object):
     def _get_this_instance(cls, *args, **kwargs):
         return cls(*args, **kwargs)
 
-    def kalman_filter(self,
-                      df: pd.Series,
-                      iters: int = 5,
-                      plot: bool = True) -> pd.Series:
-        filtered: np.ndarray
-        k_filter: KalmanFilter = KalmanFilter()
-        df: pd.Series
-        i: int
-        filtered = k_filter.filter(np.array(df))[0]
-        for i in range(iters):
-            filtered = k_filter.smooth(filtered)[0]
-        if plot:
-            self.fig.add_trace(
-                Line(
-                    name='kalman filter',
-                    y=filtered.T[0],
-                    line=dict(width=utils.SUB_LINES_WIDTH)), 1, 1)
-        return pd.Series(filtered.T[0])
-
-    def scipy_filter(self,
-                     df: pd.Series,
-                     window_length: int = 101,
-                     polyorder: int = 3,
-                     plot: bool = True,
-                     **scipy_savgol_filter_kwargs) -> pd.Series:
-        filtered = signal.savgol_filter(
-            df,
-            window_length=window_length,
-            polyorder=polyorder,
-            **scipy_savgol_filter_kwargs)
-        if plot:
-            self.fig.add_trace(
-                Line(
-                    name='savgol filter',
-                    y=filtered,
-                    line=dict(width=utils.SUB_LINES_WIDTH)), 1, 1)
-        return pd.Series(filtered)
-
     def __get_stop_take(self, sig: utils.PREDICT_TYPE) -> Dict[str, float]:
         """
         calculating stop loss and take profit.
@@ -648,7 +608,7 @@ class Trader(object):
 
     def crossover(self, fast: Iterable, slow: Iterable):
         self.returns = []
-        for s, f in zip(slow, fast):
+        for s, f in zip(list(slow), list(fast)):
             if s < f:
                 self.returns.append(utils.BUY)
             elif s > f:
@@ -741,18 +701,19 @@ class Trader(object):
                 next_l) in enumerate(zip(self.returns[:-1],
                                          self._stop_losses[:-1],
                                          self._take_profits[:-1],
-                                      seted_[:-1],
+                                         seted_[:-1],
                                          self._credit_leverages[:-1],
-                                      data_high[:-1],
-                                      data_low[:-1],
-                                      data_high[1:],
-                                      data_low[1:])):
+                                         data_high[:-1],
+                                         data_low[:-1],
+                                         data_high[1:],
+                                         data_low[1:])):
 
             if seted is not np.nan:
                 if oldsig != utils.EXIT:
                     commission_reuse = 2
                 else:
                     commission_reuse = 1
+                bet = start_bet
                 if bet > deposit:
                     bet = deposit
                 open_price = data_column[e]
@@ -766,10 +727,13 @@ class Trader(object):
                 ignore_breakout = True
 
             next_breakout = min(stop_loss, take_profit) < next_l <= next_h < max(stop_loss, take_profit)
-            if (min(stop_loss, take_profit) < low <= high < max(stop_loss, take_profit) and next_breakout) or ignore_breakout:
+            if (min(stop_loss, take_profit) < low <= high < max(stop_loss,
+                                                                take_profit) and next_breakout) or ignore_breakout:
                 diff = data_column[e + 1] - data_column[e]
             else:
                 exit_take_stop = True
+                if next_breakout:
+                    e += 1
                 if sig == utils.BUY and high >= take_profit:
                     diff = take_profit - data_column[e]
 
@@ -784,6 +748,8 @@ class Trader(object):
 
                 else:
                     diff = 0.0
+                if next_breakout:
+                    e -= 1
 
             if sig == utils.SELL:
                 diff = -diff
@@ -1211,7 +1177,7 @@ winrate: {self.winrate}%"""
 
                     self.client.order_create(predict,
                                              self.ticker,
-                                             bet*credit_leverage)
+                                             bet * credit_leverage)
                     self.__exit_order__ = False
         return {
             'predict': predict,
@@ -1585,10 +1551,10 @@ winrate: {self.winrate}%"""
     def get_support_resistanse(self) -> Dict[str, Dict[int, float]]:
         lows = self.df['Low'].values
         highs = self.df['High'].values
-        for i in range(2, len(lows)-2):
-            if lows[i-2] >= lows[i-1] >= lows[i] <= lows[i+1] <= lows[i+2]:
+        for i in range(2, len(lows) - 2):
+            if lows[i - 2] >= lows[i - 1] >= lows[i] <= lows[i + 1] <= lows[i + 2]:
                 self.supports[i] = lows[i]
-            if highs[i-2] <= highs[i-1] <= highs[i] >= highs[i+1] >= highs[i+2]:
+            if highs[i - 2] <= highs[i - 1] <= highs[i] >= highs[i + 1] >= highs[i + 2]:
                 self.resistances[i] = highs[i]
         return {'resistance': self.resistances,
                 'supports': self.supports}
